﻿using System;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace post8237
{
    public class MyOverlay : Overlay
    {
        private static Random random = new Random();
        private InMemoryMarkerOverlay markerOverlay1;
        private InMemoryMarkerOverlay markerOverlay2;

        private string imageUriString;
        private string toolTip;

        public MyOverlay(string imageUriString, string toolTip)
            : base()
        {
            this.imageUriString = imageUriString;
            this.toolTip = toolTip;

            markerOverlay1 = new InMemoryMarkerOverlay();
            markerOverlay2 = new InMemoryMarkerOverlay();

            SetStyle();

            InitRandomPoints(markerOverlay1);
            InitRandomPoints(markerOverlay2);

            OverlayCanvas.Children.Add(markerOverlay1.OverlayCanvas);
            OverlayCanvas.Children.Add(markerOverlay2.OverlayCanvas);
        }

        private void InitRandomPoints(InMemoryMarkerOverlay overlay)
        {
            overlay.FeatureSource.BeginTransaction();
            for (int i = 0; i < 20; i++)
            {
                overlay.FeatureSource.AddFeature(new Feature(random.Next(-100, 100), random.Next(-50, 50)));
            }
            overlay.FeatureSource.CommitTransaction();
        }

        private void SetStyle()
        {
            markerOverlay1.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ImageSource = new BitmapImage(new Uri(imageUriString, UriKind.RelativeOrAbsolute));
            markerOverlay1.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.Width = 16;
            markerOverlay1.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.Height = 16;
            markerOverlay1.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ToolTip = toolTip + " 16x16";
            markerOverlay1.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            markerOverlay2.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ImageSource = new BitmapImage(new Uri(imageUriString, UriKind.RelativeOrAbsolute));
            markerOverlay2.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.Width = 32;
            markerOverlay2.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.Height = 32;
            markerOverlay2.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ToolTip = toolTip + " 32x32";
            markerOverlay2.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        }

        public Projection Projection
        {
            set
            {
                markerOverlay1.FeatureSource.Projection = value;
                markerOverlay2.FeatureSource.Projection = value;
            }
        }

        protected override void DrawCore(RectangleShape targetExtent, RefreshType refreshType)
        {
            markerOverlay1.MapArguments = MapArguments;
            markerOverlay2.MapArguments = MapArguments;

            markerOverlay1.Draw(targetExtent, refreshType);
            markerOverlay2.Draw(targetExtent, refreshType);
        }

        protected override void PanToCore(RectangleShape targetExtent)
        {
            markerOverlay1.MapArguments = this.MapArguments;
            markerOverlay2.MapArguments = this.MapArguments;

            markerOverlay1.PanTo(targetExtent);
            markerOverlay2.PanTo(targetExtent);
        }

        protected override RectangleShape GetBoundingBoxCore()
        {
            RectangleShape[] rects = new RectangleShape[] { markerOverlay1.GetBoundingBox(), markerOverlay2.GetBoundingBox() };
            return ExtentHelper.GetBoundingBoxOfItems(rects);
        }
    }
}
